//************************************************************************
//*                          BipGDI Win V0.9.3                           *
//*----------------------------------------------------------------------*
//*                       Header file for BipGDI Win                     *
//*                                                                      *
//*                                                         P. Basset    *
//************************************************************************


// include BipGDI only one time
#ifndef _BipGDI_
#define _BipGDI_

#include "BipNUM.h"
#include "BipDATE.h"


	using namespace std;


	// TYPEDEF
	typedef int (*pfBG_FormProc) (const string sBGFormName, const string sBGItemName,
				const string sBGMessage,
				HWND hBGCurWnd, UINT message, WPARAM wParam, LPARAM lParam);


	// STRUCTURE
	struct BG_STRUCT_WINDOW	// structure of BipGDI window
	{
		string sTitle;		// title name of the window
		HWND hBGWnd;		// window handle
		pfBG_FormProc pfFormProc; // pointer to user proc
		int iLeft;			// window position
		int iTop;
		int iWidth;
		int iHeight;
		int iXDrawPos;		// X position for drawing
		int iYDrawPos;		// Y position for drawing
		int iLineCnt;		// current line number
		int iFirstLineCnt;	// first line to display
		int iLastLineCnt;	// last line display
		bool bInvisible;	// some text is invisible
		string sFontName;	// current font name
		int iFontHeight;	// current font height
		bool bFontBold;		// bold ?
		bool bFontItalic;	// italic ?
		bool bUnderline;	// underline ?
		bool bVScrollBar;	// vertical scroll bar
		int iBackColor;		// background color
		bool bStatusBar;	// true = the window has a status bar
		bool bModal;		// true = the window is modal (default = false)
		HWND hStatusBar;	// handle of the status bar
		string sStatusBar;	// text of the status bar
		string sReturn;		// return state
	};

	struct BG_STRUCT_MENU	// structure of BipGDI menu
	{
		string sIntName;	// internal name of the menu
		string sWinName;	// window name of then menu
		bool bEnable;		// false = menu item is visible but not enable
		bool bCheck;		// true = menu item is check
		int iId;			// Id = number return when clic
	};
	
	struct BG_STRUCT_FORMITEM	// structure of BipGDI form item
	{
		string sType;		// type of the item
		string sFormat;		// format of the item
		int iLength;		// length of the item
		int iOrder;			// order of the item (smaller = first)
		HWND hHandle;		// handle of the item
		string sValue;		// default value (for STATIC = text)
		bool bNotNull;		// true : value can't be null (or empty)
		bool bEdit;			// combobox : true = user can edit value
		bool bCheck;		// true = checkbox is check
		bool bEnable;		// false = item is visible but not enable
		bool bVisible;		// false = item is invisible
		int iBackColor;		// back color
		int iTextColor;		// text color
		HBRUSH hBackBrush;	// brush for background color
		HBRUSH hTextBrush;	// brush for text color
		int iId;			// Id = GDI number of the item
		int iLeft;			// item position
		int iTop;
		int iWidth;
		int iHeight;
		string sButton;		// function of the button (ie : ok)
		string sRemark;		// remark
		string sRetType;	// return type (string,bool,int,...)
		int iTextHPos;		// horizontal text position (-1=right,0=center,1=left)
		int iTextVPos;		// vertical text position (-1=top,0=center,1=bottom)
		string sFtName;		// name of the font
		int iFtSize;		// size of the font (in pixels)
		bool bFtBold;		// bold font
		bool bFtItalic;		// italic font
		bool bFtUline;		// underline font
		int iBrdSize;		// border size
		int iBrdColor;		// border color
		string sBrdStyle;	// border style
		string sParam;		// all the parameters
		bool bReturn;		// return value : bool
		string sReturn;		// return value : string
		BP_Currency curReturn; // return value : currency
		BP_Date dtReturn;	// return value : date/time
	};

	struct BG_SCROLL_BAR	// size of scroll bar
	{
		int iHsize;
		int iVsize;
	};

	struct BG_STRUCT_ERROR	// Error structure
	{
		string sName;		// internal window name
		string sFunction;	// function in use
	};

	struct BG_INPUTBOX		// Input Box structure
	{
		string sMessage;	// message to display
		string sReturnString; // string entered by user
		bool bOk;			// user clic on OK button
		bool bCancel;		// user clic on Cancel button
		int iReturnCode;	// return code
	};


	// BipGDI define
	const int BG_MAX_EXTRACT           = 15;	// max number of substring

	// BipGDI Color define (0x00BBGGRR)
	const int BG_COL_NONE              = -1;
	const int BG_COL_BLACK             = 0x00000000;
	const int BG_COL_WHITE             = 0x00FFFFFF;
	const int BG_COL_RED               = 0x000000FF;
	const int BG_COL_BLUE              = 0x00FF0000;
	const int BG_COL_GREEN             = 0x0000FF00;
	const int BG_COL_YELLOW            = 0x0000FFFF;
	const int BG_COL_MAGENTA           = 0x00FF00FF;
	const int BG_COL_CYAN              = 0x00FFFF00;
	const int BG_COL_GREY              = 0x00BFBFBF; //0x007F7F7F;
	const int BG_COL_LIGHTGREY         = 0x00DFDFDF; //0x00BFBFBF
	const int BG_COL_DARKGREY          = 0x007F7F7F; //0x003F3F3F;

	// BipGDI Error define
	const int BG_ERR_BADWINDNAME       = 1;		// bad window name
	const int BG_ERR_REOPEN            = 2;		// window already open or BG_ResetBipGDI not done
	const int BG_ERR_BGFULL            = 3;		// BipGDI structure is full
	const int BG_ERR_NOTOPEN           = 4;		// window doesn't exists
	const int BG_ERR_MAINNOTOPEN       = 5;		// main window is not open
	const int BG_ERR_WINDOWBADNUMBER   = 6;		// windows:bad internal number
	const int BG_ERR_NOTINRANGE        = 7;		// value not in range
	const int BG_ERR_BADCOLOR          = 8;		// color not in range
	const int BG_ERR_MENUFULL		   = 9;		// menu:structure is full
	const int BG_ERR_MENUINTEMPTY	   = 10;	// menu:internal name is empty
	const int BG_ERR_MENUBADCOLNUM	   = 11;	// menu:bad column number
	const int BG_ERR_MENUBADLINENUM	   = 12;	// menu:bad line number
//	const int BG_ERR_MENUSUBNOTFOUND   = 13;	// menu:submenu not found
	const int BG_ERR_MENUINTEXISTS	   = 14;	// menu:internal already exists
	const int BG_ERR_MENUNWINEMPTY	   = 15;	// menu:window name is empty
	const int BG_ERR_MENUITEMEMPTY	   = 16;	// menu:item name is empty
	const int BG_ERR_MENUITEMNOTFOUND  = 17;	// menu:item name not found
	const int BG_ERR_FORMITEMNAMEBAD   = 18;	// form:item name is bad or empty
	const int BG_ERR_FORMBADPARAM	   = 19;	// form:parameter is bad
	const int BG_ERR_FORMITEMTYPEBAD   = 20;	// form:item type is bad
	const int BG_ERR_FORMITEMRETBAD	   = 21;	// form:item return type is bad
	const int BG_ERR_FORMITEMREDEDINE  = 22;	// form:item is already define
	const int BG_ERR_FORMBADSTRING	   = 23;	// form:item can't accept string type
	const int BG_ERR_FORMDTFMTBAD      = 24;	// form:date format is bad or missing
	const int BG_ERR_RECTBADSTYLE      = 25;	// bad rectangle style
	const int BG_ERR_NOPOINT           = 26;	// point char is forbidden

	// BipGDI Win32 Error define
	const int BG_ERR_W32REGCLASS       = 1000;	// win32 RegisterClassEx
	const int BG_ERR_W32CREATEWINDOW   = 1001;	// win32 CreateWindowEx
	const int BG_ERR_W32GETSYSMETRIC   = 1002;	// win32 GetSystemMetrics
	const int BG_ERR_W32SETWINDOWTEXT  = 1003;	// win32 SetWindowText
	const int BG_ERR_W32SYSPARAMINFO   = 1004;	// win32 SystemParametersInfo
	const int BG_ERR_W32GETCLIENTRECT  = 1005;	// win32 GetClientRect
	const int BG_ERR_W32CREATEFONT     = 1006;	// win32 CreateFont
	const int BG_ERR_W32TEXTOUT        = 1007;	// win32 TextOut
	const int BG_ERR_W32GETTEXTEXPT32  = 1008;	// win32 GetTextExtentPoint32
	const int BG_ERR_W32GETDC          = 1009;	// win32 GetDC
	const int BG_ERR_W32RELEASEDC      = 1010;	// win32 ReleaseDC
	const int BG_ERR_W32DELETEOBJECT   = 1011;	// win32 DeleteObject
	const int BG_ERR_W32MOVEWINDOW     = 1012;	// win32 MoveWindow
	const int BG_ERR_W32SELECTOBJECT   = 1013;	// win32 SelectObject
	const int BG_ERR_W32BEGINPAINT     = 1014;	// win32 BeginPaint
	const int BG_ERR_W32GLOBALALLOC    = 1015;	// win32 GlobalAlloc
	const int BG_ERR_W32GLOBALLOCK     = 1016;	// win32 GlobalLock
	const int BG_ERR_W32SHOWSCROLLBAR  = 1017;	// win32 ShowScrollBar
	const int BG_ERR_W32UPDATEWINDOW   = 1018;	// win32 UpdateWindow
	const int BG_ERR_W32INVALIDATERECT = 1019;	// win32 InvalidateRect
	const int BG_ERR_W32GLOBALUNLOCK   = 1020;  // win32 GlobalUnlock
	const int BG_ERR_W32DIALOGBOXIND   = 1021;  // win32 DialogBoxIndirect
	const int BG_ERR_W32GLOBALFREE     = 1022;  // win32 GlobalFree
	const int BG_ERR_W32SETFOCUS       = 1023;  // win32 SetFocus
	const int BG_ERR_W32GETDLTITEMTEXT = 1024;  // win32 GetDlgItemText
	const int BG_ERR_W32ENDDIALOG      = 1025;  // win32 EndDialog
	const int BG_ERR_W32SETTEXTCOLOR   = 1026;	// win32 SetTextColor
	const int BG_ERR_W32SETSCROLLPOS   = 1027;	// win32 SetScrollPos
	const int BG_ERR_W32CREATESBRUSH   = 1028;	// win32 CreateSolidBrush
	const int BG_ERR_W32SETBKCOLOR     = 1029;	// win32 SetBkColor
	const int BG_ERR_W32SETMENU		   = 1030;	// win32 SetMenu
	const int BG_ERR_W32DESTROYMENU	   = 1031;	// win32 DestroyMenu
	const int BG_ERR_W32CREATEMENU	   = 1032;	// win32 CreateMenu
	const int BG_ERR_W32APPENDMENU	   = 1033;	// win32 AppendMenu
	const int BG_ERR_W32ENABLEMENUIT   = 1034;	// win32 EnableMenuItem
	const int BG_ERR_W32CHECKMENUIT    = 1035;	// win32 CheckMenuItem
	const int BG_ERR_W32GETMODHANDLE   = 1036;	// win32 GetModuleHandle
	const int BG_ERR_W32DESTROYWINDOW  = 1037;	// win32 DestroyWindow
	const int BG_ERR_W32INITCOMMCTRL   = 1038;	// win32 InitCommonControlsEx
	const int BG_ERR_W32EXTTEXTOUT     = 1039;	// win32 ExtTextOut
	const int BG_ERR_W32DRAWEDGE       = 1040;	// win32 DrawEdge
	const int BG_ERR_W32DRAWFOCUSRECT  = 1041;	// win32 DrawFocusRect
	const int BG_ERR_W32MOVETOEX       = 1042;	// win32 MoveToEx
	const int BG_ERR_W32LINETO         = 1043;	// win32 LineTo
	const int BG_ERR_W32EXTCREATEPEN   = 1044;	// win32 ExtCreatePen
	const int BG_ERR_W32DRAWTEXT       = 1045;	// win32 DrawText
	const int BG_ERR_W32RECTANGLE      = 1046;	// win32 Rectangle
	const int BG_ERR_W32REDRAWWINDOW   = 1047;	// win32 RedrawWindow

	// BipPLUS Error define
	const int BP_ERR_NOMENU            = 9000;	// no menu found
	const int BP_ERR_NOFORM            = 9001;	// no form found


	// string for parameters in BipGDI functions
	const string BG_PRM_LF			= "LF";			// Line Feed

	// string for parameters in BipGDI Form functions
	const string BG_PRM_NAME		= "NAME";		// name of the window
	const string BG_PRM_BACKCOLOR	= "BKCOL";		// background color
	const string BG_PRM_TEXTCOLOR	= "TXCOL";		// text color
	const string BG_PRM_EDGE		= "EDGE";		// edge for the window
	const string BG_PRM_MODAL		= "MODAL";		// window is modal
	const string BG_PRM_STATUSBAR	= "STBAR";		// statusbar exist
	const string BG_PRM_ENABLE		= "ENABLE";		// menu or item is enable
	const string BG_PRM_VISIBLE		= "VISIBLE";	// item is visible
	const string BG_PRM_CHECK		= "CHECK";		// check box is check
	const string BG_PRM_VALUE		= "VALUE";		// default value of the item
	const string BG_PRM_BUTTON		= "BUTTON";		// function of the button
	const string BG_PRM_TYPE		= "TYPE";		// type of item
	const string BG_PRM_LENGTH		= "LENGTH";		// length of item result
	const string BG_PRM_FORMAT		= "FORMAT";		// format of item value
	const string BG_PRM_REMARK		= "REMARK";		// statusbar remark
	const string BG_PRM_NOTNULL		= "NOTNULL";	// item can be null
	const string BG_PRM_EDIT		= "EDIT";		// item can be modify
	const string BG_PRM_TXHPOS		= "TXHPOS";		// horizontal position of text
	const string BG_PRM_TXVPOS		= "TXVPOS";		// vertical position of text
	const string BG_PRM_FTNAME		= "FTNAME";		// name of the font for text
	const string BG_PRM_FTSIZE		= "FTSIZE";		// size of the font (in pixels)
	const string BG_PRM_FTBOLD		= "FTBOLD";		// bold font
	const string BG_PRM_FTITALIC	= "FTITALIC";	// italic font
	const string BG_PRM_FTULINE		= "FTULINE";	// underline font
	const string BG_PRM_BRDSIZE		= "BRDSIZE";	// border size
	const string BG_PRM_BRDCOL		= "BRDCOL";		// border color
	const string BG_PRM_BRDSTYLE	= "BRDSTYLE";	// border style

	// string for items in BipGDI Form functions
	const string BG_ITEM_STATIC		= "STATIC";		// static item (label)
	const string BG_ITEM_RECT		= "RECT";		// rectangle
	const string BG_ITEM_EDIT		= "EDIT";		// editbox
	const string BG_ITEM_BUTTONSYS	= "BUTTONSYS";	// system button
	const string BG_ITEM_BUTTON		= "BUTTON";		// ownerdraw button
	const string BG_ITEM_CHECKSYS	= "CHECKSYS";	// system check button
	const string BG_ITEM_COMBOSYS	= "COMBOSYS";	// system combobox
	const string BG_ITEM_DATAGRID	= "DATAGRID";	// data grid

	// item return type in BipGDI Form functions
	const string BG_RET_STRING		= "STRING";
	const string BG_RET_DATE		= "DATE";
	const string BG_RET_CURRENCY	= "CURRENCY";

	// item message in BipGDI Form functions
	const string BG_MSG_BEFORE		= "BEFORE";
	const string BG_MSG_ITEMCHANGE	= "ITEMCHG";
	const string BG_MSG_ITEMEXIT	= "ITEMEXIT";
	const string BG_MSG_ITEMCLIC	= "ITEMCLIC";


	// Forward declarations of functions included in this code module:

	// WINDOW functions
	int BG_WindowMainCreate (const string sBGWindowName, const int nCmdShow,
						const WNDPROC wndproc, const string sBGParam = "",
						const int iBGLeft = 0, const int iBGTop = 0,
						const int iBGWidth = 0, const int iBGHeight = 0);
						// sBGParam = "", iBGLeft = 0, iBGTop = 0, iBGWidth = 0, iBGHeight = 0
	int BG_WindowChildCreate (const string sBGChildName, const WNDPROC childwndproc,
						 const string sBGParam = "", const int iBGLeft = 0,
						 const int iBGTop = 0, const int iBGWidth = 0,
						 const int iBGHeight = 0);
						// sBGParam = "", iBGLeft = 0, iBGTop = 0, iBGWidth = 0,
						// iBGHeight = 0
	int BG_WindowSelect (const string sBGName);
	int BG_WindowRelease (void);
	int BG_WindowCls (void);
	BG_STRUCT_WINDOW BG_WindowGet (const string sBGName);
	BG_STRUCT_WINDOW BG_WindowNullGet (void);
	int BG_WindowTitleSet (const string sBGName, const string sBGWndTitle);
	int BG_WindowMove (const string sBGName, const int iBGLeft, const int iBGTop,
						const int iBGWidth, const int iBGHeight);
	int BG_WindowRedraw (void);

	int BG_WindowMessageAuto (const UINT message, const WPARAM wParam);
	UINT BG_MessageLoop (void);

	// BOX functions
	int BG_MessageBox (const string sBGTitle, const string sBGText);
	string BG_InputBox (const string sBGText, const string sBGTitle = "",
						const string sBGDefValue = "");
	int BG_InputBoxErrorGet (void);
	int CALLBACK BG_InputBoxProcIntl (HWND hBGCurWnd, UINT message, WPARAM wParam,
									LPARAM lParam);

	// MENU functions
	int BG_MenuAdd (const string sBGIntName, const int iBGColumn, const int iBGLine,
				const string sBGWinName, const string sBGParam = "");
				// sParam = ""
	int BG_MenuDisplay (void);
	int BG_MenuNumGet (const int iBGId);
	int BG_MenuNumGet (const string sBGIntName);
	int BG_MenuItemEnable (const string sBGIntName, const bool bBGEnable);
	int BG_MenuItemCheck (const string sBGIntName, const bool bBGCheck);
	BG_STRUCT_MENU BG_MenuGet (const int iBGColumn, const int iBGLine);
	BG_STRUCT_MENU BG_MenuGet (const string sBGIntName);
	string BG_MenuMessage (const UINT message, const WPARAM wParam);

	// FORM functions
	int BG_FormCreate (const string sBGFormName, const string sBGParam,
				   const int iBGLeft, const int iBGTop, const int iBGWidth,
				   const int iBGHeight, pfBG_FormProc = NULL);
				// = NULL
	int BG_FormClose (const string sBGFormName);
	string BG_FormReturnValueGet (const string sBGFormName);
	int BG_FormCreateItem (const HWND hBGCurWnd, const string sBGFormName,
					   const string sBGItemName);
	int BG_FormItemAdd (const string sBGFormName, const string sBGItemName,
					 const string sBGItemType, const int iBGOrder,
					const int iBGLeft, const int iBGTop, const int iBGWidth,
					const int iBGHeight, const string sBGParam = "");
					// sBGParam = ""
	int BG_FormItemModify (const string sBGFormName, const string sBGItemName,
					   const string sBGParam, const string sBGValue);
	int BG_FormItemRedraw (const string sBGFormName, const string sBGItemName);
	int BG_FormItemFontSet (const HDC hBGdevice, const string sBGFontName, const int iBGFontHeight,
				const bool bBGBold = false, const bool bBGItalic = false,
				const bool bBGUnderline = false);
				// bBGBold = false, bBGItalic = false, bBGUnderline = false

	BG_STRUCT_FORMITEM BG_FormItemGet (const string sBGFormName, const string sBGItemName);
	string BG_FormItemNameGet (const string sBGFormName, const int iBGId);
	string BG_FormItemNameGet (const string sBGFormName, const HWND hBGHandle);
	BG_STRUCT_FORMITEM BG_FormItemNullGet ();
	string BG_FormItemValueGetString (const string sBGFormName, const string sBGItemName);
	BOOL CALLBACK BG_FormProcIntl (HWND hBGCurWnd, UINT message, WPARAM wParam, LPARAM lParam);

	int BG_RectangleDraw (const HDC hBGdevice, const int iBGLeft, const int iBGTop,
					  const int iBGRight, const int iBGBottom, const string sBGStyle,
					  const int iBGLineSize = 0, const int iBGColor = 0,
					  const int iBackColor = -1);
				// iBGLineSize = 0, iBGColor = 0, iBackColor = -1

	// text functions
	int BG_TextDraw (const string sBGText, const string sBGParam = "");
				// sParam = ""

	int BG_TextFontSet (const string sBGFontName, const int iBGFontHeight,
				const int iBGBold = -1, const int iBGItalic = -1, const int iBGUnderline = -1);
				// iBGBold = -1, iBGItalic = -1, iBGUnderline = -1
	int BG_TextPositionSet (const int iBGHPos);
	int BG_TextColorSet (const string sBGColor);

	// other functions
	int BG_ResetBipGDIIntl (void);

	void BG_ErrorSet (const string sBGName, const string sBGFunction);
	string BG_ErrorDisplay (const int iBGError);
	void BG_ErrorDisplaySet (const bool bBGError = true); // bBGError = true
	void BG_ErrorSave (void);
	void BG_ErrorRestore (void);

	string BG_StringToUpper (string sBGString);
	int BG_StringToInt (const string sString, const int iBase = 10);
				// iBase = 10
	string BG_IntToString (const int iInt);
	string BG_StringParamGetInt (const string sBGParam, const string sBGString);
	bool BG_StringIsTrue (const string sBGInput);
	bool BG_StringIsFalse (const string sBGInput);
	int BG_ColorNumGet (const string sBGColor);


#endif /* _BipGDI_ */